// SPDX-License-Identifier: GPL-3.0-only
mod crypto;
mod error;
mod utils;

use clap::{Arg, ArgMatches, Command};

use crypto::*;
use error::FcryError;

fn build_cmdline_args() -> ArgMatches {
    Command::new("fcry - [f]ile[cry]pt").version(env!("CARGO_PKG_VERSION"))
        .about("A file en-/decryption tool for easy use.")
        .arg(Arg::new("decrypt")
            .short('d')
            .long("decrypt")
            .help("Decrypt instead of encrypt. Encrypting is the default.")
            .takes_value(false))
        .arg(Arg::new("input_file")
            .short('i')
            .long("input-file")
            .help("The input file to en-/decrypt")
            .takes_value(true))
        .arg(Arg::new("output_file")
            .short('o')
            .long("output-file")
            .help("The output file. If not specified, en-/decrypted bytes will be sent to stdout.")
            .takes_value(true))
        .arg(Arg::new("raw-key")
            .required(true)
            .long("raw-key")
            .help("The raw bytes of the crypto key. Has to be exactly 32 bytes.\n*** DANGEROUS, use for testing purposes only! ***")
            .takes_value(true))
        .get_matches()
}

fn run(args: ArgMatches) -> Result<(), FcryError> {
    let input_file = args.value_of("input_file");
    let output_file = args.value_of("output_file");

    let mut key = [0u8; 32];
    key.clone_from_slice(args.value_of("raw-key").unwrap().as_bytes());

    if args.is_present("decrypt") {
        decrypt(input_file, output_file, key)?
    } else {
        encrypt(input_file, output_file, key)?
    }

    Ok(())
}

fn main() {
    if let Err(e) = run(build_cmdline_args()) {
        println!("Error: {:?}", e);
    }
}
